class Fit:
    import pandas as pd
    from .analyzers import StressStrain

    def __init__(self, data: StressStrain, cut_off: bool = True, fit_window: int = 200):

        self.data = data.data
        self.max_stress_index = self.maxStressIndex()

        if cut_off:
            self.cutOffData()

        self.fit_results = self.getBestFit(fit_window)
        self.results = self.getResults()

    def maxStressIndex(self):
        max_stress_row = self.data[self.data['stress']
                                   == self.data['stress'].max()]
        return max_stress_row.index[0]

    def getResults(self) -> pd.DataFrame:
        import pandas as pd

        maxValues = self.data.iloc[self.max_stress_index]
        yieldValues = self.getYield()
        return pd.DataFrame({
            'Max Stress [kPa]': [maxValues['stress']],
            'Max Strain': [maxValues['strain']],
            'Young Modulus [kPa]': [self.fit_results.slope],
            'Intercept [kPa]': [self.fit_results.intercept],
            'Yield Stress [kPa]': [yieldValues['stress']],
            'Yield Strain': [yieldValues['strain']], })

    def cutOffData(self):
        self.data = self.data.iloc[0:self.max_stress_index + 1]
        return None

    def getBestFit(self, window=200):
        import pandas as pd
        import numpy as np

        window = 200
        step = 10

        fit_results = pd.DataFrame()

        sizeData = len(self.data[self.data['strain'] < 0.1])

        if window / sizeData > 0.2:
            window = max([int(sizeData * 0.2), min([30, sizeData])])
            step = 5

        left = 0
        right = window

        while True:

            if right > sizeData:
                break

            data = self.data[left:right]

            slope, intercept = np.polyfit(data['strain'], data['stress'], 1)

            y_exp = slope * data['strain'] + intercept

            ss_res = np.sum((data['stress'] - y_exp) ** 2)
            ss_tot = np.sum((data['stress'] - np.mean(data['stress'])) ** 2)
            r2 = 1 - (ss_res / ss_tot)

            error = (data['stress'] - y_exp)**2
            df = pd.DataFrame({
                'strain':  data[['strain']].iloc[int(
                    window / 2)].values,
                'slope': slope,
                'intercept': intercept,
                'error': error.sum(),
                'r2': r2
            })
            fit_results = pd.concat([fit_results, df], axis=0)

            left += step
            right += step
        bestFit = fit_results[fit_results['slope']
                              == fit_results['slope'].max()].iloc[0]
        print('Best Fit \n')
        print(bestFit)

        return bestFit

    def getYield(self):
        df = self.data.copy()
        df['error'] = abs(df['stress'] - (df['strain']-0.02)
                          * self.fit_results.slope - self.fit_results.intercept)
        yieldValues = df[df['error'] == df['error'].min()].iloc[0]
        return yieldValues



class TRAFolder:
    class FolderNotFound(Exception):
        pass

    def __init__(self, folder_path: str):
        self.folder_path = folder_path

        self.file_list = self.getTRAFiles()

        import pandas as pd
        self.results = pd.DataFrame()

        import os
        if not os.path.exists(folder_path + '/plots'):
            os.makedirs(folder_path + '/plots')

    def analyze(self, options={}):
        if not self.file_list:
            return

        import pandas as pd

        for file in self.file_list:
            print(file)
            result = TRAFile(file, self.folder_path).analyze(options)
            result['File'] = self.pad_numeric_part(file)

            self.results = pd.concat([self.results, result])

        self.printCopyrights()
        return self.results.set_index('File').sort_index()

    def getTRAFiles(self):
        import os

        try:
            dir_list = os.listdir(self.folder_path)

            try:
                file_list = [
                    filename for filename in dir_list if filename.endswith('.TRA')]

                if not file_list:
                    print('No .TRA files found in the folder.')
                    return []
                else:
                    return file_list

            except FileNotFoundError:
                print('An error occurred while filtering files.')
                raise

        except FileNotFoundError:
            raise TRAFolder.FolderNotFound('Folder not found')

    def pad_numeric_part(self, filename):
        parts = filename.split('_')
        if len(parts) != 2:
            return filename
        prefix, numeric_part = parts
        try:
            numeric_part = int(numeric_part.split('.')[0])
            padded_numeric_part = '{:04d}'.format(numeric_part)
            return '{}_{}.{}'.format(prefix, padded_numeric_part, filename.split('.')[-1])
        except ValueError:
            return filename

    def printCopyrights(self):
        print("*********************************************")
        print("                EXTRUDION")
        print("*********************************************")
        print()
        print("by Paride Azzari (C) 2024")
        print()
        print("info on: github.com/azzarip/extrudion")
        print("*********************************************")
        print("RESULTS:")
        print()
        print("Analysis.csv contains the analyzed data")
        print()
        print("The plots folder contains all the figures")
        print("*********************************************")
        print()


class TRAFile:

    def __init__(self, file: str, folder_path: str):
        import os
        import pandas as pd
        self.filename = file.replace('.TRA', '')

        if folder_path:
            self.filepath = os.path.join(folder_path, file)
        else:
            self.filepath = file

        df = pd.read_table(self.filepath, header=[
                           3], encoding='Windows-1252', sep=',')
        df['mm'] = df['mm'].apply(replace_negative_values)
        self.data = df

    def analyze(self, options):
        from .analyzers import StressStrain
        from .fit import Fit
        from .plot import Plot

        data = StressStrain(
            self.data, sample_area=options['sample_area'], initial_length=options['initial_length'])

        fit = Fit(data)

        if "plot" not in options or options["plot"] is not False:
            Plot(fit, data, self.filename)

        print(fit.results)
        return fit.results


def replace_negative_values(x):
    if x >= 0:
        return x
    if abs(x) < 0.01:
        return 0
    else:


<< << << < HEAD
raise ValueError("Negative values in column 'mm'")
== == == =
raise ValueError("Negative values in column 'mm'")
>>>>>> > 8c1643b1469747eec8636596ff5bf458efe2392c
